use image::imageops::FilterType;
use image::{DynamicImage, GenericImageView};

use crate::config::image_processors::ProcessorParams;
use crate::result::MBGResult;

pub struct ScaleProcessor {
    pub params: ProcessorParams,
    pub screen_height: i32,
    pub screen_width: i32,
}

impl super::ImageProcessor for ScaleProcessor {
    fn process(&self, img: &DynamicImage) -> MBGResult<DynamicImage> {
        let abs_width = self.screen_width.abs() as u32;
        let scale_factor = ((abs_width / img.width()) as f32) * self.params.strength;
        let scale_factor = scale_factor.ceil() as u32;
        debug!("Scaling image by {} times", scale_factor);
        Ok(img.resize(
            img.width() * scale_factor,
            img.height() * scale_factor,
            FilterType::Nearest,
        ))
    }
}
