#[macro_use]
extern crate failure;
#[macro_use]
extern crate failure_derive;
#[macro_use]
extern crate log;
#[macro_use]
extern crate serde_derive;

use structopt::StructOpt;

use crate::config::Config;
use crate::logging::setup_logger;
use crate::player_dbus::start_listen;
use crate::result::{MBGError, MBGResult};

pub mod background;
pub mod config;
pub mod dbus_interface;
pub mod display;
pub mod img_processors;
pub mod logging;
pub mod player_dbus;
pub mod result;

#[derive(Debug, StructOpt)]
#[structopt(
    name = "music_bg",
    about = "Listens to your mpris interface and sets you a cool background."
)]
struct Opt {
    #[structopt(subcommand)]
    cmd: Option<RunMode>,
}

#[derive(StructOpt, Debug)]
enum RunMode {
    #[structopt(name = "run", about = "Run d-bus listener")]
    Run,
    #[structopt(
        name = "config",
        about = "Create or replace default configuration file."
    )]
    GenConf,
}

fn main() -> MBGResult<()> {
    let opt: Opt = Opt::from_args();
    let mode = opt.cmd.unwrap_or_else(|| RunMode::Run);
    let logger = setup_logger();
    if let Err(error) = logger {
        error!("Failed to create pretty logger");
        error!("{}", error.to_string());
        error!("Using default logging");
    }
    let res = gtk::init();
    if res.is_err() {
        return Err(MBGError::X11Error(String::from("Could not connect to GTK")));
    }

    match mode {
        RunMode::Run => start_listen(),
        RunMode::GenConf => Config::generate_config(),
    }
}
